=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2009-01-27 07:48:55 +0000
+++ bzrlib/smart/request.py	2009-02-11 09:45:27 +0000
@@ -104,7 +104,8 @@
         
         Must return a SmartServerResponse.
         """
-        raise NotImplementedError(self.do_body)
+        if body_bytes != '':
+            raise SmartProtocolError('Request does not expect a body.')
 
     def do_chunk(self, chunk_bytes):
         """Called with each body chunk if the request has a streamed body.

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2009-01-15 09:01:42 +0000
+++ bzrlib/tests/__init__.py	2009-02-11 09:45:27 +0000
@@ -2874,6 +2874,7 @@
                    'bzrlib.tests.test_shelf_ui',
                    'bzrlib.tests.test_smart',
                    'bzrlib.tests.test_smart_add',
+                   'bzrlib.tests.test_smart_request',
                    'bzrlib.tests.test_smart_transport',
                    'bzrlib.tests.test_smtp_connection',
                    'bzrlib.tests.test_source',

=== added file 'bzrlib/tests/test_smart_request.py'
--- bzrlib/tests/test_smart_request.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_smart_request.py	2009-02-11 09:45:27 +0000
@@ -0,0 +1,41 @@
+# Copyright (C) 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for smart server request infrastructure (bzrlib.smart.request)."""
+
+from bzrlib.smart import request
+from bzrlib.tests import TestCase
+
+
+class TestSmartRequest(TestCase):
+
+    def test_request_class_without_do_body(self):
+        """If a request has no body data, and the request's implementation does
+        not override do_body, then no exception is raised.
+        """
+        # Create a SmartServerRequestHandler with a SmartServerRequest subclass
+        # that does not implement do_body.
+        class NoBodyRequest(request.SmartServerRequest):
+            """A request that does not implement do_body."""
+            def do(self):
+                return request.SuccessfulSmartServerResponse(('ok',))
+        handler = request.SmartServerRequestHandler(
+            None, {'foo': NoBodyRequest}, '/')
+        # Emulate a request with no body (i.e. just args).
+        handler.args_received(('foo',))
+        handler.end_received()
+        # no exception was raised.
+

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2009-02-11 07:06:42 +0000
+++ bzrlib/tests/test_smart_transport.py	2009-02-11 09:45:27 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

